@@ -75,16 +75,38 @@ namespace coder_compiler
     };
   }
 
-  int IndentingOStreambuf::overflow( int ch )
+  IndentingOStreambuf::int_type IndentingOStreambuf::overflow( IndentingOStreambuf::int_type ch )
   {
-    if ( myIsAtStartOfLine && ch != '\n' )
+        if (traits_type::eq_int_type(ch, traits_type::eof())) {
+            return traits_type::not_eof(ch);
+        }
+
+        if (myIsAtStartOfLine && !traits_type::eq_int_type(ch, traits_type::to_int_type('\n'))) {
+            myDest->sputn(myIndent.data(), myIndent.size());
+        }
+
+        myIsAtStartOfLine = traits_type::eq_int_type(ch, traits_type::to_int_type('\n'));
+        return myDest->sputc(traits_type::to_char_type(ch));
+  }
+
+  std::streamsize IndentingOStreambuf::xsputn(const char* s, std::streamsize n)
+  {
+    std::streamsize written = 0;
+
+    for (std::streamsize i = 0; i < n; ++i)
       {
-        myDest->sputn( myIndent.data(), myIndent.size() );
+        if (traits_type::eq_int_type(overflow(traits_type::to_int_type(s[i])), traits_type::eof()))
+          {
+              break;
+          }
+        ++written;
       }
+    return written;
+  }
 
-    myIsAtStartOfLine = ch == '\n';
-
-    return myDest->sputc( ch );
+  int IndentingOStreambuf::sync()
+  {
+    return myDest->pubsync();
   }
 
   void IndentingOStreambuf::release ()
@@ -343,7 +365,7 @@ namespace coder_compiler
   code_generator::visit_anon_fcn_handle (octave::tree_anon_fcn_handle& anon_fh)
   {
     os_src
-      << "Anonymous (fcn2ov(["
+      << "Anonymous (sfcn2ov (new auto (std::move (["
       << "="
       << "](coder_value_list& output, const octave_value_list& args, int nargout) mutable\n{\n";
 
@@ -399,7 +421,7 @@ namespace coder_compiler
 
     decrement_indent_level (os_src);
 
-    os_src << "}))";
+    os_src << "}))))";
 
     fcn_scopes.pop_back ();
   }
@@ -956,9 +978,31 @@ namespace coder_compiler
 #endif
 
   void
-  code_generator::visit_octave_user_script (octave_user_script&)
+  code_generator::visit_octave_user_script (octave_user_script& scr)
   {
-    warning("user script is not supported!");
+    fcn_scopes.push_back (traversed_scopes.front()[0].front());
+
+    os_src
+      << "Symbol (fcn2ov(["
+      << "](coder_value_list& output, const octave_value_list& args, int nargout)\n{\n";
+
+    increment_indent_level (os_src);
+
+    declare_and_define_variables();
+
+    octave_function * f = scr.function_value ();
+
+    os_src
+      << mangle(f->name ())
+      << ".evaluate (nargout);\n";
+
+    os_src << "return make_return_list(output);\n";
+
+    decrement_indent_level (os_src);
+
+    os_src << "}))";
+
+    fcn_scopes.pop_back ();
   }
 
   void
@@ -990,7 +1034,7 @@ namespace coder_compiler
       {
         //print_comment_list (leading_comment);
       }
-#endif    
+#endif
 
     bool nested  = fcn.is_nested_function() ;
 
@@ -1109,8 +1153,23 @@ namespace coder_compiler
 
     auto scope_searcher = traversed_scopes.front()[0].front();
 
+    const auto& variables = scope[(int)symbol_type::ordinary];
+
+    set_symbols_t scripts, scripts_variables;
+    const set_symbols_t * symbols;
+
+    std::tie (scripts, scripts_variables) = find_script_variables (scope_searcher);
+
+    if (! scripts_variables.empty ())
+      {
+        scripts_variables.insert(variables.begin(), variables.end ());
+        symbols = &scripts_variables;
+      }
+    else
+      symbols = &variables;
+
     os_src
-      << "ConstCast (Symbol (fcn2ov (ConstCast ([=";
+      << "ConstCast (Symbol (sfcn2ov (new auto ([=";
 
     os_src
       << "](coder_value_list& output, const octave_value_list& args, int nargout) mutable\n{\n";
@@ -1119,7 +1178,7 @@ namespace coder_compiler
 
 		declare_persistent_variables ();
 
-    for(const auto& symbol : scope[(int)symbol_type::ordinary])
+    for(const auto& symbol : *symbols)
       {
         os_src
           << "Symbol "
@@ -1141,7 +1200,7 @@ namespace coder_compiler
                 )
                   {
                   }
-                else if (symbol->file )
+                else if (symbol->file && symbol->file->type != file_type::script)
                   {
                     os_src
                       << " = Copy (";
@@ -1157,7 +1216,7 @@ namespace coder_compiler
                   }
               }
           }
-        else if (symbol->file )
+        else if (symbol->file && symbol->file->type != file_type::script)
           {
             os_src
               << " = Copy (";
@@ -1191,6 +1250,8 @@ namespace coder_compiler
 					<< ([name]  () mutable {for (auto & c: name) c = toupper(c); return name;})()
 					<< "_MAKER;\n";
 			}
+
+		define_scripts (scripts);
   }
   void
   code_generator::visit_octave_user_function_trailer (octave_user_function& fcn)
@@ -1250,6 +1311,35 @@ namespace coder_compiler
       }
   }
 
+  void
+  code_generator::visit_function_def (octave::tree_function_def& fdef)
+  {
+    octave_value fcn = fdef.function ();
+
+    octave_function *f = fcn.function_value ();
+
+    if (f)
+      {
+        auto name = f->name ();
+
+        os_src
+          << mangle (name)
+          << " = ";
+
+        os_src
+          << "Copy (";
+
+        os_src
+          << mangle(current_script->name)
+          << current_script->id;
+
+        os_src
+          << "::"
+          << mangle(name)
+          << "make());\n";
+      }
+  }
+
   void
   code_generator::visit_identifier (octave::tree_identifier&  id )
   {
@@ -1412,7 +1502,7 @@ namespace coder_compiler
                   }
                 else
                   {
-                    std::string text_rep = quote(fn) + "__";
+                    std::string text_rep = quote(fn) + "_sq";
 
                     auto f = constant_map.find(text_rep);
 
@@ -1576,7 +1666,7 @@ namespace coder_compiler
                   << "\""
                   << row
                   << "\""
-                  << (sq ? "__" : "_dq") ;
+                  << (sq ? "_sq" : "_dq") ;
 
                 if (nstr > 1)
                   os << "}" ;
@@ -1596,20 +1686,20 @@ namespace coder_compiler
         os
           << "Colon ("
           << r.base()
-          << "__, "
+          << "_d, "
           << r.increment()
-          << "__, "
+          << "_d, "
           << r.limit()
-          << "__)";
+          << "_d)";
 #else
          os
           << "Colon ("
           << r.base()
-          << "__, "
+          << "_d, "
           << r.inc()
-          << "__, "
+          << "_d, "
           << r.limit()
-          << "__)";
+          << "_d)";
 #endif
       }
     else if ( m_value.is_real_scalar() )
@@ -1618,7 +1708,7 @@ namespace coder_compiler
           {
           case btyp_double:
             {
-              os << m_value.double_value() << "__";
+              os << m_value.double_value() << "_d";
 
               break;
             }
@@ -1696,7 +1786,7 @@ namespace coder_compiler
 
             for (octave_idx_type j = 0; j < nc; j++)
               {
-                os << m.elem(i,j) << "__";
+                os << m.elem(i,j) << "_d";
 
                 if (j < nc - 1)
                   os << ", ";
@@ -1844,7 +1934,7 @@ namespace coder_compiler
 
         for (auto& str : fields)
           {
-            std::string text_rep = quote(str) + "__";
+            std::string text_rep = quote(str) + "_sq";
 
             auto f = constant_map.find(text_rep);
 
@@ -1898,6 +1988,8 @@ namespace coder_compiler
 
     bool is_resolved = true;
 
+    bool is_script = false;
+
     auto idx = visit_dot_separated_fcn_handle (fh);
 
     if (! idx)
@@ -1913,6 +2005,8 @@ namespace coder_compiler
 
             if (is_resolved && symbol->file)
               {
+                is_script = symbol->file->type == file_type::script;
+
                 os_src
                   << "&"
                   << mangle(symbol->file->name)
@@ -1921,12 +2015,18 @@ namespace coder_compiler
               }
           }
 
-        os_src
-          << mangle(fh.name());
-
-        if (is_resolved && ! is_special_function && ! is_nested)
+        if (is_script)
           os_src
-            << "make";
+            << "script_entry";
+        else
+          {
+            os_src
+              << mangle(fh.name());
+
+            if (is_resolved && ! is_special_function && ! is_nested)
+              os_src
+                << "make";
+          }
       }
 
     os_src
@@ -2580,6 +2680,58 @@ namespace coder_compiler
     return str + "_";
   }
 
+  void
+  code_generator::define_scripts(const set_symbols_t& scripts)
+  {
+		auto saved_script = current_script;
+
+		for (const auto& symbol : scripts)
+      {
+        current_script = symbol->file;
+
+        os_src
+          << mangle (symbol->name)
+          << " = ";
+
+        os_src
+          << "Symbol (sfcn2ov (new auto ([&";
+
+        os_src
+          << "](coder_value_list& output, const octave_value_list& args, int nargout)\n{\n";
+
+        increment_indent_level (os_src);
+
+        os_src
+          << "if (nargout > 0)\n";
+
+        increment_indent_level (os_src);
+
+        os_src
+          << "call_error (\"invalid call to script "
+          << symbol->name
+          << "\");\n";
+
+        decrement_indent_level (os_src);
+
+        symbol->fcn.user_script_value()->body ()->accept(*this);
+
+        os_src << "Return:\n";
+
+        increment_indent_level (os_src);
+
+        os_src << "return make_return_list(output);\n";
+
+        decrement_indent_level (os_src);
+
+        decrement_indent_level (os_src);
+
+        os_src
+          << "})));\n";
+      }
+
+    current_script = saved_script;
+  }
+
   void
   code_generator::declare_and_define_variables()
   {
@@ -2589,6 +2741,22 @@ namespace coder_compiler
 
     auto scope_searcher = traversed_scopes.front()[0].front();
 
+    const auto& variables = scope[(int)symbol_type::ordinary];
+
+    set_symbols_t scripts, scripts_variables;
+    const set_symbols_t * symbols;
+
+    std::tie (scripts, scripts_variables) = find_script_variables (scope_searcher);
+
+    if (! scripts_variables.empty ())
+      {
+        scripts_variables.insert(variables.begin(), variables.end ());
+        symbols = &scripts_variables;
+      }
+    else
+      symbols = &variables;
+
+
     static const std::map<std::string,std::string> special_functions ({
       {"nargin", "NARGIN"},
       {"nargout", "NARGOUT"},
@@ -2597,7 +2765,7 @@ namespace coder_compiler
       {"nargoutchk", "NARGOUTCHK"}
     });
 
-    for(const auto& symbol : scope[(int)symbol_type::ordinary])
+    for(const auto& symbol : *symbols)
       {
         os_src
           << "Symbol "
@@ -2619,7 +2787,7 @@ namespace coder_compiler
                 )
                   {
                   }
-                else if (symbol->file )
+                else if (symbol->file && symbol->file->type != file_type::script)
                   {
                     os_src
                       << " = Copy (";
@@ -2635,7 +2803,7 @@ namespace coder_compiler
                   }
               }
           }
-        else if (symbol->file )
+        else if (symbol->file && symbol->file->type != file_type::script)
           {
             os_src
               << " = Copy (";
@@ -2664,13 +2832,14 @@ namespace coder_compiler
 
     for (auto& name : special_names)
 			{
-			  if (scope_searcher->contains(name))
+			  if (scope_searcher->contains(name) || symbols-> count (std::make_shared<coder_symbol>(name)))
 				os_src
 					<< ([name]  () mutable {for (auto & c: name) c = toupper(c); return name;})()
 					<< "_MAKER;\n";
 			}
-  }
 
+		define_scripts (scripts);
+  }
 
   void
   code_generator::declare_and_define_handle_variables()
@@ -2680,6 +2849,20 @@ namespace coder_compiler
     const auto current_scope = traversed_scopes.front()[1].front();
 
     const auto& scope = current_scope->symbols();
+    const auto& variables = scope[(int)symbol_type::ordinary];
+
+    set_symbols_t scripts, scripts_variables;
+    const set_symbols_t * symbols;
+
+    std::tie (scripts, scripts_variables) = find_script_variables (current_scope);
+
+    if (! scripts_variables.empty ())
+      {
+        scripts_variables.insert(variables.begin(), variables.end ());
+        symbols = &scripts_variables;
+      }
+    else
+      symbols = &variables;
 
     traversed_scopes.front()[1].pop_front();
 
@@ -2691,7 +2874,7 @@ namespace coder_compiler
       {"nargoutchk", "NARGOUTCHK"}
     });
 
-    for(const auto& symbol : scope[(int)symbol_type::ordinary])
+    for(const auto& symbol : *symbols)
       {
         os_src
           << "Symbol "
@@ -2713,7 +2896,7 @@ namespace coder_compiler
                 )
                   {
                   }
-                else if (symbol->file)
+                else if (symbol->file && symbol->file->type != file_type::script)
                   {
                     os_src
                       << " = Copy (";
@@ -2729,7 +2912,7 @@ namespace coder_compiler
                   }
               }
           }
-        else if (symbol->file )
+        else if (symbol->file && symbol->file->type != file_type::script)
           {
             os_src
               << " = Copy (";
@@ -2763,6 +2946,8 @@ namespace coder_compiler
 					<< ([name]  () mutable {for (auto & c: name) c = toupper(c); return name;})()
 					<< "_MAKER;\n";
 			}
+
+		define_scripts (scripts);
   }
 
   void
@@ -2883,6 +3068,51 @@ namespace coder_compiler
       }
   }
 
+  std::pair<set_symbols_t, set_symbols_t>
+  code_generator::find_script_variables(const symscope_ptr& scope)
+  {
+    set_symbols_t new_scripts, new_script_variables;
+
+    const auto& symbols = scope->symbols()[(int)symbol_type::ordinary];
+
+    for(const auto& symbol : symbols)
+      {
+        if (symbol->file)
+          {
+            if (symbol->file->type == file_type::script)
+              {
+                if (! m_scripts.count(symbol))
+                  {
+                    m_scripts.insert (symbol);
+
+                    new_scripts.insert(symbol);
+
+                    const auto& script_scope = symbol->file->local_functions[0].first.current_scope ();
+
+                    const auto& symbols_in_script = script_scope->symbols ()[(int)symbol_type::ordinary];
+
+                    for(const auto& symbol : symbols_in_script)
+                      {
+                        if (! current_function_scope->contains(symbol) && ! current_function_scope->lookup_in_parent_scopes(symbol) && ! m_scripts_variables.count(symbol))
+                          {
+                            m_scripts_variables.insert (symbol);
+
+                            new_script_variables.insert (symbol);
+                          }
+                      }
+
+                    auto tmp = find_script_variables(script_scope);
+
+                    new_scripts.insert (tmp.first.begin (), tmp.first.end ());
+                    new_script_variables.insert (tmp.second.begin (), tmp.second.end ());
+                  }
+              }
+          }
+      }
+
+    return {new_scripts, new_script_variables};
+  }
+
   void
   code_generator::generate_header(  )
   {
@@ -2906,7 +3136,7 @@ namespace coder_compiler
         for (const auto& var : m_file->local_functions)
           os_hdr
             << "Symbol& "
-            << mangle(var.name())  << "make();\n";
+            << mangle(var.first.name())  << "make();\n";
       }
     else if(m_file->type == file_type::m)
       {
@@ -2935,6 +3165,36 @@ namespace coder_compiler
               }
           }
       }
+    else if(m_file->type == file_type::script)
+      {
+        os_hdr
+          << "const Symbol& "
+          << "script_entry ();\n";
+        bool first = true;
+
+        for (const auto& sub: m_file->local_functions)
+          {
+            if (first)
+              {
+                first = false;
+                continue;
+              }
+            const auto& nm = sub.first.name();
+
+            const auto& f = sub.second;
+
+            octave_function *fcn = f.function_value (true);
+
+            if (fcn && fcn->is_user_function())
+              {
+                octave_user_function& ufc = *(fcn->user_function_value());
+
+                os_hdr
+                  << "const Symbol& "
+                  << mangle(nm) << "make ();\n";
+              }
+          }
+      }
     else if(m_file->type == file_type::oct)
       {
         os_hdr << "const Symbol& " << mangle(m_file->name) << "make();\n";
@@ -2943,7 +3203,7 @@ namespace coder_compiler
           {
             os_hdr
               << "const Symbol& "
-              << mangle(var.name())  << "make();\n";
+              << mangle(var.first.name())  << "make();\n";
           }
       }
     else
@@ -2968,10 +3228,11 @@ namespace coder_compiler
 
     increment_indent_level (os_prt);
 
-    os_prt
-      << "const Symbol& "
-      << mangle(m_file->name)
-      << "make(){Symbol* sym = nullptr; return *sym;}\n";
+    if(m_file->type != file_type::script)
+      os_prt
+        << "const Symbol& "
+        << mangle(m_file->name)
+        << "make(){Symbol* sym = nullptr; return *sym;}\n";
 
     if(m_file->type == file_type::m)
       for (const auto& sub: m_file->fcn.user_function_value()->subfunctions())
@@ -2995,6 +3256,37 @@ namespace coder_compiler
             }
           }
         }
+    else if (m_file->type == file_type::script)
+      {
+        os_hdr
+          << "const Symbol& "
+          << "script_entry {Symbol* sym = nullptr; return *sym;}\n";
+
+        bool first = true;
+
+        for (const auto& sub: m_file->local_functions)
+          {
+            if (first)
+              {
+                first = false;
+                continue;
+              }
+            const auto& nm = sub.first.name();
+
+            const auto& f = sub.second;
+
+            octave_function *fcn = f.function_value (true);
+
+            if (fcn && fcn->is_user_function())
+              {
+                octave_user_function& ufc = *(fcn->user_function_value());
+
+                os_hdr
+                  << "const Symbol& "
+                  << mangle(nm) << "make(){Symbol* sym; return *sym;}\n";
+              }
+          }
+      }
 
     decrement_indent_level (os_prt);
 
@@ -3037,11 +3329,39 @@ namespace coder_compiler
     };
     if (mode != gm_compact)
       {
+        std::unordered_set<coder_file_ptr> scriptdeps;
+
+        const auto& dependency = dependency_graph.at(m_file);
+
+        std::function <void(const std::unordered_set<coder_file_ptr>&)> get_dep;
+
+        get_dep = [&](const std::unordered_set<coder_file_ptr>& depends)
+        {
+          for (const auto&  file: depends)
+            {
+
+              if (! scriptdeps.count (file) && ! dependency.count(file))
+                {
+                  scriptdeps.insert(file);
+                }
+
+              if (file->type == file_type::script)
+                {
+                  get_dep (dependency_graph.at (file));
+                }
+            }
+        };
+
+        get_dep (dependency);
+
         os_src << "#include " << quote("coder.h") << "\n";
 
         inclusion(m_file);
 
-        for ( const auto&  dep: dependency_graph.at(m_file))
+        for ( const auto&  dep: dependency)
+          inclusion(dep);
+
+        for ( const auto&  dep: scriptdeps)
           inclusion(dep);
 
         os_src << "using namespace coder;\n";
@@ -3060,16 +3380,16 @@ namespace coder_compiler
           {
             os_src
               << "Symbol& "
-              << mangle(var.name())
+              << mangle(var.first.name())
               << "make()\n{";
 
             increment_indent_level (os_src);
 
             os_src
               << "static Symbol "
-              << mangle(var.name())
+              << mangle(var.first.name())
               << "; return "
-              << mangle(var.name())  ;
+              << mangle(var.first.name())  ;
 
             decrement_indent_level (os_src);
 
@@ -3105,26 +3425,36 @@ namespace coder_compiler
           quote(undo_string_escapes1(m_file->path)));
 
         for (const auto& var : m_file->local_functions)
-          define_oct(mangle(var.name()), quote(var.name()), quote(m_file->name),
+          define_oct(mangle(var.first.name()), quote(var.first.name()), quote(m_file->name),
             quote(undo_string_escapes1(m_file->path)));
       }
     else
       {
-        if (m_file->type == file_type::m || m_file->type == file_type::cmdline)
+        if (m_file->type == file_type::m || m_file->type == file_type::cmdline || m_file->type == file_type::script)
           {
             os_src << "static Constant& Const(int);\n";
           }
 
-        os_src
-          << "const Symbol& "
-          << mangle(m_file->name)
-          << "make()\n{\n";
+        if (m_file->type != file_type::script)
+          os_src
+            << "const Symbol& "
+            << mangle(m_file->name)
+            << "make()\n{\n";
+        else
+          os_src
+            << "const Symbol& "
+            << "script_entry ()\n{\n";
 
         increment_indent_level (os_src);
 
-        os_src
-          << "static const Symbol "
-          << mangle(m_file->name) ;
+        if (m_file->type != file_type::script)
+          os_src
+            << "static const Symbol "
+            << mangle(m_file->name);
+        else
+          os_src
+            << "static const Symbol "
+            << "script";
 
         os_src << "(";
 
@@ -3141,6 +3471,10 @@ namespace coder_compiler
                 nested_fcn_names.push_back (mangle(m_file->name));
               }
 
+            current_function_scope = traversed_scopes.front()[0].front();
+            m_scripts.clear ();
+            m_scripts_variables.clear ();
+
             fcn->accept(*this);
 
             if (has_nested)
@@ -3150,6 +3484,19 @@ namespace coder_compiler
 
             nesting_context -= has_nested;
           }
+        else if (m_file->type == file_type::script)
+          {
+
+            current_function_scope = traversed_scopes.front()[0].front();
+            m_scripts.clear ();
+            m_scripts_variables.clear ();
+
+            current_script = m_file;
+
+            octave_user_script *scr = m_file->fcn.user_script_value (true);
+
+            scr->accept(*this);
+          }
         else
           {
             os_src
@@ -3178,9 +3525,14 @@ namespace coder_compiler
               }
           }
 
-        os_src
-          << ");\nreturn "
-          << mangle(m_file->name) ;
+        if (m_file->type != file_type::script)
+          os_src
+            << ");\nreturn "
+            << mangle(m_file->name);
+        else
+          os_src
+            << ");\nreturn "
+            << "script";
 
         os_src  << ";\n";
 
@@ -3225,6 +3577,10 @@ namespace coder_compiler
                       os_src
                         << "(";
 
+                      current_function_scope = traversed_scopes.front()[0].front();
+                      m_scripts.clear ();
+                      m_scripts_variables.clear ();
+
                       fcn->accept(*this);
 
                       os_src
@@ -3244,8 +3600,78 @@ namespace coder_compiler
                     }
                 }
             }
+        else if (m_file->type == file_type::script )
+          {
+            bool first = true;
 
-        if (m_file->type == file_type::m || m_file->type == file_type::cmdline)
+            for (const auto& sub: m_file->local_functions)
+              {
+                if (first)
+                  {
+                    first = false;
+                    continue;
+                  }
+
+                const auto& nm = sub.first.name();
+
+                const auto& f = sub.second;
+
+                octave_function *fcn = f.function_value (true);
+
+                if (fcn && fcn->is_user_function())
+                  {
+                    octave_user_function& ufc = *(fcn->user_function_value());
+
+                    if (!ufc.is_nested_function())
+                      {
+                        bool has_nested = has_nested_function (ufc);
+
+                        nesting_context += has_nested;
+
+                        if (has_nested)nested_fcn_names.push_back (mangle(nm));
+
+                        traversed_scopes.pop_front();
+
+                        os_src
+                          << "const Symbol& "
+                          << mangle(nm)
+                          << "make()\n{\n";
+
+                        increment_indent_level (os_src);
+
+                        os_src
+                          << "static const Symbol "
+                          << mangle(nm) ;
+
+                        os_src
+                          << "(";
+
+                        current_function_scope = traversed_scopes.front()[0].front();
+                        m_scripts.clear ();
+                        m_scripts_variables.clear ();
+
+                        fcn->accept(*this);
+
+                        os_src
+                          << ");\nreturn ";
+
+                        os_src
+                          << mangle(nm)
+                          << ";\n";
+
+                        decrement_indent_level (os_src);
+
+                        os_src << "}\n";
+
+                        if (has_nested) nested_fcn_names.pop_back ();
+
+                        nesting_context -= has_nested;
+                      }
+                  }
+              }
+          }
+
+        if (m_file->type == file_type::m || m_file->type == file_type::cmdline || m_file->type == file_type::script)
           {
             os_src
               << "static Constant& Const(int i)\n{\n";
