@@ -17,12 +17,16 @@
 #include <octave/pt-walk.h>
 #include <octave/version.h>
 
+#include "coder_symtab.h"
+
 class octave_user_function;
 
 class octave_value;
 
 namespace coder_compiler
 {
+  using set_symbols_t = std::set<coder_symbol_ptr, symscope::compare_symbol>;
+
   enum  loop_or_unwind_state
   {
     normal_context,
@@ -35,7 +39,9 @@ namespace coder_compiler
   //https://stackoverflow.com/a/9600752/6579744
   protected:
 
-    virtual int overflow( int ch );
+    virtual int_type overflow(int_type ch) override;
+    virtual std::streamsize xsputn(const char* s, std::streamsize n) override;
+    virtual int sync() override;
 
   public:
 
@@ -186,7 +192,7 @@ namespace coder_compiler
     visit_octave_user_function_trailer (octave_user_function& fcn);
 
     void
-    visit_function_def (octave::tree_function_def&){}
+    visit_function_def (octave::tree_function_def&);
 
     void
     visit_identifier (octave::tree_identifier&  id );
@@ -281,13 +287,14 @@ namespace coder_compiler
 
     void
     visit_do_until_command (octave::tree_do_until_command& cmd);
+#if OCTAVE_MAJOR_VERSION >= 6
+    void
+    visit_superclass_ref (octave::tree_superclass_ref&) {}
+
+    void
+    visit_metaclass_query (octave::tree_metaclass_query&) {}
+#endif
 #if OCTAVE_MAJOR_VERSION >= 8
-    void
-    visit_superclass_ref(octave::tree_superclass_ref&){}
-
-    void
-    visit_metaclass_query(octave::tree_metaclass_query&){}
-
     void
     visit_classdef_attribute(octave::tree_classdef_attribute&){}
 
@@ -352,6 +359,9 @@ namespace coder_compiler
     std::string
     mangle (const std::string& str);
 
+    void
+    define_scripts (const set_symbols_t&);
+
     void
     declare_and_define_variables();
 
@@ -370,8 +380,8 @@ namespace coder_compiler
     void
     declare_persistent_variables();
 
-    void
-    reset();
+    std::pair <set_symbols_t, set_symbols_t>
+    find_script_variables(const symscope_ptr& scope);
 
     void
     generate_header( );
@@ -431,5 +441,14 @@ namespace coder_compiler
     std::vector<std::string> nested_fcn_names;
 
     std::vector<symscope_ptr > fcn_scopes;
+
+    coder_file_ptr current_script;
+
+    symscope_ptr current_function_scope;
+
+    set_symbols_t m_scripts;
+
+    set_symbols_t m_scripts_variables;
+
   };
 }
