@@ -84,7 +84,7 @@ namespace coder_compiler
     { }
 
     void
-    visit_return_list (octave::tree_return_list& lst){}
+    visit_return_list (octave::tree_return_list& lst)
     { }
 #endif
     void
@@ -179,13 +179,13 @@ namespace coder_compiler
 
     void
     visit_return_command (octave::tree_return_command&) {}
-
+#if OCTAVE_MAJOR_VERSION >= 6
     void
     visit_superclass_ref (octave::tree_superclass_ref&) {}
 
     void
     visit_metaclass_query (octave::tree_metaclass_query&) {}
-
+#endif
     void
     visit_classdef_attribute (octave::tree_classdef_attribute&) {}
 
